/*
 * JSVG Library
 * 
 *   Copyright (C) 2020, Jonathan Sevy <jsevy@jsevy.com>
 *   
 *   Permission is hereby granted, free of charge, to any person obtaining a copy
 *   of this software and associated documentation files (the "Software"), to deal
 *   in the Software without restriction, including without limitation the rights
 *   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the Software is
 *   furnished to do so, subject to the following conditions:
 *   
 *   The above copyright notice and this permission notice shall be included in all
 *   copies or substantial portions of the Software.
 *   
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *   SOFTWARE.
 * 
 */


package com.jsevy.jsvg;

import java.util.Vector;

import com.jsevy.jxml.XMLElement;


/**
 * A container for the font faces used in the document. At present just presents three fonts to which the
 * generic Java serif, san-serif and monospaced fonts are mapped.
 * @author jsevy
 *
 */
class SVGFontFaces extends XMLElement
{
    
    private Vector<SVGFontFace> fonts = new Vector<SVGFontFace>();
    
    
    /**
     * Create a font faces container object, and populate with three basic Liberation fonts (Liberation Serif,
     * Liberation Sans, Liberation Mono) that should be present on any platform supporting Open Document Foundation documents.
     */
    public SVGFontFaces()
    {
        fonts = new Vector<SVGFontFace>();
        
        // add standard LibreOffice fonts
        SVGFontFace  serif = new SVGFontFace("Liberation Serif", "&apos;Liberation Serif&apos;", "roman", "variable");
        SVGFontFace  sanSerif = new SVGFontFace("Liberation Sans", "&apos;Liberation Sans&apos;", "roman", "variable");
        SVGFontFace  mono = new SVGFontFace("Liberation Mono", "&apos;Liberation Mono&apos;", "modern", "fixed");
        
        fonts.add(serif);
        this.appendChild(serif);
        
        fonts.add(sanSerif);
        this.appendChild(sanSerif);
        
        fonts.add(mono);
        this.appendChild(mono);
        
        populateXML();
    }
    
    
    private void populateXML()
    {
        this.setNodeName("font-face-decls");
    }
    
    
    /**
     * Map Java logical fonts Serif, SanSerif and Monospaced to their corresponding Liberation fonts.
     * @param javaName  Java logical font name - Serif, SanSerif, Monospaced
     * @return          ODG font name Liberation Serif, Liberation Sans, or Liberation Mono.
     */
    public static String mapJavaFontName(String javaName)
    {
        if (javaName.equalsIgnoreCase("Serif"))
            return "Liberation Serif";
        else if (javaName.equalsIgnoreCase("SanSerif"))
            return "Liberation Sans";
        else if (javaName.equalsIgnoreCase("Monospaced"))
            return "Liberation Mono";
        else 
            return "Liberation Sans";
    }
    
    
    
}
