package com.jsevy.jxml;

import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMConfiguration;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;

public class XMLDocument extends XMLNode implements Document
{
    
    protected Element documentElement;
    
    protected DocumentType documentType = null;
    
    protected String inputEncoding = null;
    protected String xmlEncoding = null;
    protected String xmlVersion = null;
    protected boolean xmlStandalone = false;
    protected boolean strictErrorChecking = false;
    protected String documentURI = null;
    
    
    public XMLDocument(String rootTag)
    {
        // node name of Document node is "#document"
        super("#document");
        
        this.documentElement = new XMLElement(rootTag);
        this.appendChild(documentElement);
    }
    
    
    public XMLDocument()
    {
        // will want to set root element name subsequently
        this("");
    }
    
    
    /*
    public void setRootElementName(String rootTag)
    {
        this.documentElement.setNodeName(rootTag);
    }
    
    
    public String getRootElementName()
    {
        return this.documentElement.getNodeName();
    }
    */
    
    
    @Override
    public Node cloneNode(boolean deep)
    {
        // create new document fro
        XMLDocument document = new XMLDocument();
        
        document.inputEncoding = this.inputEncoding;
        document.xmlEncoding = this.xmlEncoding;
        document.xmlVersion = this.xmlVersion;
        document.xmlStandalone = this.xmlStandalone;
        document.strictErrorChecking = this.strictErrorChecking;
        document.documentURI = this.documentURI;
        
        // clone document type element
        if (this.documentType != null)
            document.documentType = (XMLDocumentType)this.documentType.cloneNode(deep);
        
        // clone document element, set in new document
        document.setDocumentElement((XMLElement)this.documentElement.cloneNode(deep));
        
        return document;
    }
    
    
    
    @Override
    public short getNodeType()
    {
        return Node.DOCUMENT_NODE;
    }
    
    
    @Override
    public DocumentType getDoctype()
    {
        return this.documentType;
    }
    
    
    public void setDoctype(DocumentType documentType)
    {
        this.documentType = documentType;
    }
    
    
    
    @Override
    public DOMImplementation getImplementation()
    {
        // TODO Auto-generated method stub
        return null;
    }
    
    
    
    @Override
    public Element getDocumentElement()
    {
        return documentElement;
    }
    
    
    public void setDocumentElement(Element newDocumentElement)
    {
        Element oldDocumentElement = this.documentElement;
        this.replaceChild(newDocumentElement, oldDocumentElement);
        this.documentElement = newDocumentElement;
    }
    
    
    public String getTextContent() throws DOMException
    {
        // return depends on subclass
        // return is null for DOCUMENT_NODE, DOCUMENT_TYPE_NODE, NOTATION_NODE,
        return null;
    }
    
    
    
    @Override
    public Element createElement(String tagName) throws DOMException
    {
        XMLElement element = new XMLElement(tagName);
        element.setOwnerDocument(this, false);
        return element;
    }
    
    
    
    @Override
    public DocumentFragment createDocumentFragment()
    {
        // TODO Auto-generated method stub
        return null;
    }
    
    
    
    @Override
    public Text createTextNode(String data)
    {
        XMLText text = new XMLText(data);
        text.setOwnerDocument(this, false);
        return text;
    }
    
    
    
    @Override
    public Comment createComment(String data)
    {
        XMLComment comment = new XMLComment(data);
        comment.setOwnerDocument(this, false);
        return comment;
    }
    
    
    
    @Override
    public CDATASection createCDATASection(String data) throws DOMException
    {
        XMLCDATASection cDataSection = new XMLCDATASection(data);
        cDataSection.setOwnerDocument(this, false);
        return cDataSection;
    }
    
    
    
    @Override
    public ProcessingInstruction createProcessingInstruction(String target, String data) throws DOMException
    {
        // TODO Auto-generated method stub
        return null;
    }
    
    
    
    @Override
    public Attr createAttribute(String name) throws DOMException
    {
        XMLAttribute attribute = new XMLAttribute(name);
        attribute.setOwnerDocument(this, false);
        return attribute;
    }
    
    
    
    @Override
    public EntityReference createEntityReference(String name) throws DOMException
    {
        // TODO Auto-generated method stub
        return null;
    }
    
    
    
    @Override
    public NodeList getElementsByTagName(String tagname)
    {
        return documentElement.getElementsByTagName(tagname);
    }
    
    
    
    @Override
    public Node importNode(Node importedNode, boolean deep) throws DOMException
    {
        // basically a clone operation, with ownerDocument set to this in the clones
        Node clone = importedNode.cloneNode(deep);
        
        // set owner document for everything, if possible...
        if (clone instanceof XMLNode)
        {
            ((XMLNode)clone).setOwnerDocument(this, deep);
        }
        else
        {
            throw new DOMException(DOMException.NOT_SUPPORTED_ERR, "Could not set owner document when importing Node");
        }
        
        return clone;
    }
    
    
    
    @Override
    public Element createElementNS(String namespaceURI, String qualifiedName) throws DOMException
    {
        XMLElement element = new XMLElement(qualifiedName);
        element.setNamespaceURI(namespaceURI);
        element.setOwnerDocument(this, false);
        return element;
    }
    
    
    
    @Override
    public Attr createAttributeNS(String namespaceURI, String qualifiedName) throws DOMException
    {
        XMLAttribute attribute = new XMLAttribute(qualifiedName);
        attribute.setNamespaceURI(namespaceURI);
        attribute.setOwnerDocument(this, false);
        return attribute;
    }
    
    
    
    @Override
    public NodeList getElementsByTagNameNS(String namespaceURI, String localName)
    {
        NodeList elements = documentElement.getElementsByTagNameNS(namespaceURI, localName);
        return elements;
    }
    
    
    
    @Override
    public Element getElementById(String elementId)
    {
        // would prefer recursion, but not possible since this isn't an Element method
        // luckily can get full list of descendants with getElementsByTag("*")
        NodeList descendants = this.getDocumentElement().getElementsByTagName("*");
        
        for (int i = 0; i < descendants.getLength(); i++)
        {
            Node node = descendants.item(i);
            if (node.getNodeType() == Node.ELEMENT_NODE)
            {
                NamedNodeMap attributes = node.getAttributes();
                for (int j = 0; j < attributes.getLength(); j++)
                {
                    Attr attribute = (Attr)attributes.item(j);
                    if ((attribute.isId()) && (attribute.getValue().equals(elementId)))
                        return (Element)node;
                }
            }
        }
        
        // no child with specified id; return null
        return null;
    }
    
    
    
    @Override
    public String getInputEncoding()
    {
        return this.inputEncoding;
    }
    
    
    public void setInputEncoding(String inputEncoding)
    {
        this.inputEncoding = inputEncoding;
    }
    
    
    
    @Override
    public String getXmlEncoding()
    {
        return this.xmlEncoding;
    }
    
    
    public void setXmlEncoding(String xmlEncoding)
    {
        this.xmlEncoding = xmlEncoding;
    }
    
    
    
    @Override
    public boolean getXmlStandalone()
    {
        return this.xmlStandalone;
    }
    
    
    
    @Override
    public void setXmlStandalone(boolean xmlStandalone) throws DOMException
    {
        this.xmlStandalone = xmlStandalone;
    }
    
    
    
    @Override
    public String getXmlVersion()
    {
        return this.xmlVersion;
    }
    
    
    
    @Override
    public void setXmlVersion(String xmlVersion) throws DOMException
    {
        this.xmlVersion = xmlVersion;
    }
    
    
    
    @Override
    public boolean getStrictErrorChecking()
    {
        return this.strictErrorChecking;
    }
    
    
    
    @Override
    public void setStrictErrorChecking(boolean strictErrorChecking)
    {
        this.strictErrorChecking = strictErrorChecking;
    }
    
    
    
    @Override
    public String getDocumentURI()
    {
        return this.documentURI;
    }
    
    
    
    @Override
    public void setDocumentURI(String documentURI)
    {
        this.documentURI = documentURI;
    }
    
    
    
    @Override
    public Node adoptNode(Node source) throws DOMException
    {
        if (source instanceof XMLNode)
        {
            // set parent to null, and owner document to this for node and all children and attributes
            ((XMLNode)source).setParentNode(null);
            
            // well this is easy! method is recursive already
            ((XMLNode)source).setOwnerDocument(this, true);
        }
        else
        {
            throw new DOMException(DOMException.NOT_SUPPORTED_ERR, "Could not set parent and owner document when adopting Node");
        }
        
        return source;
    }
    
    
    
    @Override
    public DOMConfiguration getDomConfig()
    {
        // TODO Auto-generated method stub
        return null;
    }
    
    
    
    @Override
    public void normalizeDocument()
    {
        this.getDocumentElement().normalize();
    }
    
    
    
    @Override
    public Node renameNode(Node n, String namespaceURI, String qualifiedName) throws DOMException
    {
        // TODO Auto-generated method stub
        return null;
    }
    
}
