package com.jsevy.jxml;

import org.w3c.dom.DOMException;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Entity;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.Notation;

public class XMLDocumentType extends XMLNode implements DocumentType
{
    
    
    
    protected String publicId = null;
    protected String systemId = null;
    protected String internalSubset = null;
    
    protected XMLNamedNodeMap entities = new XMLNamedNodeMap();
    protected XMLNamedNodeMap notations = new XMLNamedNodeMap();
    
    
    public XMLDocumentType(String name, String value)
    {
        super(name, value);
    }
    
    
    public XMLDocumentType(String name)
    {
        super(name);
    }
    
    
    public XMLDocumentType()
    {
        super();
    }
    
    
    
    @Override
    public short getNodeType()
    {
        // TODO Auto-generated method stub
        return Node.DOCUMENT_TYPE_NODE;
    }
    
    
    
    @Override
    public Node cloneNode(boolean deep)
    {
        XMLDocumentType newNode = new XMLDocumentType(this.getNodeName(), this.getNodeValue());
        
        // clone entities
        for (int i = 0; i < entities.getLength(); i++)
        {
            XMLEntity newEntity = (XMLEntity)entities.item(i).cloneNode(deep);
            newNode.entities.setNamedItem(newEntity);
        }
        
        // clone notations
        for (int i = 0; i < notations.getLength(); i++)
        {
            XMLNotation notation = (XMLNotation)notations.item(i).cloneNode(deep);
            newNode.notations.setNamedItem(notation);
        }
        
        
        if (deep)
        {
            // clone kids recursively - do we ever have kids? I dunno...
            for (int i = 0; i < this.getChildNodes().getLength(); i++)
            {
                Node child = this.getChildNodes().item(i);
                Node newChild = child.cloneNode(deep);
                newNode.appendChild(newChild);
            }
        }
        
        return newNode;
    }
    
    
    
    @Override
    public String getTextContent() throws DOMException
    {
        // TODO Auto-generated method stub
        return null;
    }
    
    
    
    @Override
    public String getName()
    {
        return this.getNodeName();
    }
    
    
    
    @Override
    public NamedNodeMap getEntities()
    {
        return this.entities;
    }
    
    
    public void addEntity(Entity entity)
    {
        entities.setNamedItem(entity);
    }
    
    
    
    @Override
    public NamedNodeMap getNotations()
    {
        return this.notations;
    }
    
    
    public void addNotation(Notation notation)
    {
        notations.setNamedItem(notation);
    }
    
    
    
    @Override
    public String getPublicId()
    {
        return this.publicId;
    }
    
    
    public void setPublicId(String publicId)
    {
        this.publicId = publicId;
    }
    
    
    
    @Override
    public String getSystemId()
    {
        return this.systemId;
    }
    
    
    public void setSystemId(String systemId)
    {
        this.systemId = systemId;
    }
    
    
    
    @Override
    public String getInternalSubset()
    {
        return this.internalSubset;
    }
    
    
    public void setInternalSubset(String internalSubset)
    {
        this.internalSubset = internalSubset;
    }
    
}
